/*
* $Log$
* Revision 2.3  2004/02/22 15:17:04  richq
* Added code to get just friendly chars at a location. Also guidance
* is now recorded.
*
* Revision 2.2  2004/02/18 19:26:54  richq
* Character info is not held in tables as per the speccy version -
* instead easier-to-follow structs are used.
* Routines here for finding who is at a location, character tiredness,
* checking the current status, recruiting, and creating the "character
* reports" shown on info screens.
*
* Revision 2.1.1.1  2004/02/04 21:49:21  Default
* Doomdark's Revenge GBA
*
*
*/
#include "stdlib.h"
#include "characters.h"
#include "main.h"
#include "army.h"
#include "objects.h"
#include "tokens.h"
#include "tokendata.h"
#include "flames.h"
#include "infoscreen.h"
#include "gfx.h"
#include "savearea.h"
#include "guidance.h"

#define TABLE_SIZE    128
#define CHAR_LOC_X              (TABLE_SIZE * 10)
#define CHAR_LOC_Y              (TABLE_SIZE * 11)
#define CHAR_FLAGS_1            (TABLE_SIZE * 12)
#define CHAR_FLAGS_2            (TABLE_SIZE * 13)
#define CHAR_FLAGS_3            (TABLE_SIZE * 14)
#define CHAR_ENERGY             (TABLE_SIZE * 15)
#define CHAR_FIGHTING           (TABLE_SIZE * 16)
#define CHAR_FOE                (TABLE_SIZE * 17)
#define CHAR_LIEGE              (TABLE_SIZE * 18)
#define CHAR_RECKLESS           (TABLE_SIZE * 19)
#define CHAR_OBJECT             (TABLE_SIZE * 20)
#define CHAR_ATTRIBUTE_GOOD     (TABLE_SIZE * 21)
#define CHAR_ATTRIBUTE_BAD      (TABLE_SIZE * 22)
#define CHAR_DESPOND            (TABLE_SIZE * 23)


int g_unloyals_here;
int g_total_here;
//int g_selectedCharID;
//character_t * g_selectedChar;
character_t * g_characters;
int g_gameover;
u8 * g_charactersHere;

//void setSelectedChar(int i) {
//  g_selectedChar = &g_characters[i];
//  g_selectedCharID = i;
////  g_saveArea.selectedChar = i;
//}


// initialisation of character data in RAM
void init_characters(void) {
  if (g_characters == 0) {
    g_characters = (character_t*)malloc(128 * sizeof(character_t));
    g_charactersHere = (u8*) malloc(128);
    
//    gbaprintf("sizeof(character_t) %d",sizeof(character_t));
  }

  // fill characterss 
  // copies from tablesInRom to make life easier afterwards
  int i;
  for (i = 0; i < 128; i++) {
    
    g_characters[i].location.x          = tablesInRom[i+CHAR_LOC_X         ];
    g_characters[i].location.y          = tablesInRom[i+CHAR_LOC_Y         ];
    g_characters[i].time_looking        = tablesInRom[i+CHAR_FLAGS_1       ];
    g_characters[i].race_orders         = tablesInRom[i+CHAR_FLAGS_2       ];
    g_characters[i].flags               = tablesInRom[i+CHAR_FLAGS_3       ];
    g_characters[i].energy              = tablesInRom[i+CHAR_ENERGY        ];
    g_characters[i].fighting            = tablesInRom[i+CHAR_FIGHTING      ];
    g_characters[i].foe                 = tablesInRom[i+CHAR_FOE           ];
    g_characters[i].liege               = tablesInRom[i+CHAR_LIEGE         ];
    g_characters[i].reckless            = tablesInRom[i+CHAR_RECKLESS      ];
    g_characters[i].object              = tablesInRom[i+CHAR_OBJECT        ];
    g_characters[i].attribute_good      = tablesInRom[i+CHAR_ATTRIBUTE_GOOD];
    g_characters[i].attribute_bad       = tablesInRom[i+CHAR_ATTRIBUTE_BAD ];
    g_characters[i].despondence         = tablesInRom[i+CHAR_DESPOND       ];
    
  }
  
  g_saveArea.selectedChar = LUXOR;
}


void getCharactersAt(const location_t * pos) {
  
  //7092...
  g_unloyals_here = 0;
  g_total_here = 0;
  int i;
  
  for ( i = 0; i < 128; i++) {
    // loop over chars
    if (g_characters[i].reckless != 0 && 
    g_characters[i].location.x == pos->x &&
    g_characters[i].location.y == pos->y && 
    g_status.charInTunnel == IN_TUNNEL(g_characters[i].flags) ) {
      // not dead, at this location and has the same tunnellyness
      // store in working area
      g_charactersHere[g_total_here++] = i;
      if (g_status.loyalty != LOYALTY(g_characters[i].flags) ) {
        g_unloyals_here++;
      }
    }
    
  }
  
  if (g_status.charInTunnel) {
    // armyhere = 0
    g_garrisonHere = 0;
  }
  else {
    // work out garrisons here value..
    // call 79d8
    calculateGarrisonAt(pos);
  }
  
  
}


// used for the map screen to find just friends
void getAllFriendsAt(const location_t * pos) {
  
  g_total_here = 0;
  g_status.loyalty = LOYALTY(g_characters[g_saveArea.selectedChar].flags);
  int i;
  
  
  for ( i = 0; i < 128; i++) {
    // loop over chars...
    if (g_characters[i].reckless != 0 && 
    g_characters[i].location.x == pos->x &&
    g_characters[i].location.y == pos->y &&
    g_status.loyalty == LOYALTY(g_characters[i].flags) ) {
      // not dead, at this location and a friend
      g_charactersHere[g_total_here++] = i;
    }
    
  }
  
}


// how tired each race gets when doing stuff
const u8 raceTiredness[16] = {
  1, 2, 2, 4, 0, 1, 2, 3, 2, 4, 1, 2, 1, 2, 0, 3,
};

void increaseU8By(u8 * val, int increment) {
  int newVal = *val + increment;
  *val = newVal>0xff?0xff:newVal;
}
void decreaseU8By(u8 * val, int decrement) {
  int newVal = *val - decrement;
  *val = newVal<0?0:newVal;
}


void makeCharacterTired(int tire) {
  // get the base tiredness
  int sapEnergy = raceTiredness[g_characters[tire].race_orders&0xf];
  if ( IS_STRONG(g_characters[tire].attribute_good) )
    sapEnergy += 2;
  else if ( IS_WEAK(g_characters[tire].attribute_bad) )
    sapEnergy += 4;
  else
    sapEnergy += 3;
  
  decreaseU8By(&g_characters[tire].energy, sapEnergy);
  
}


// used all over the place this one:
// 87b4
// checks if we are alive.
// then works out what is at our current location immediately activates it
// then checks for game over (luxor's death)
// then checks for character's death and either draws the dead screen, 
// the "check place" screen or returns normally.
int checkLocationAndStatus(void) {
  
  if (g_characters[g_saveArea.selectedChar].reckless) {
    // is alive 87c4
    workOutLocationDetails(g_saveArea.selectedChar);
    if (g_localDetails.thing_here != OBJECT_EMPTY) {
      // do the effect of the thing here:
      activateObject(g_localDetails.thing_here);
    }
    if (g_characters[LUXOR].reckless == 0) {
      // game over!!!!
      lostGame();
      return 1;
    }
    
  }
  // 87de
  if (g_characters[g_saveArea.selectedChar].reckless) {
    if (g_localDetails.thing_here != OBJECT_EMPTY/*something here*/) {
      // 84bf
      // do a screen that is like "think"
      // check place...
      infoPlace(0);
      return 1;
    }
  }
  else {
    // do dead screen
    deadScreen();
    return 1;
  }
  return 0;
}

// only char 2 can recruit morking...
void recruitMorkin(void) {
  g_characters[MORKIN].time_looking = g_characters[TARITHEL].time_looking;
  g_characters[MORKIN].foe = g_characters[TARITHEL].foe;
  g_characters[MORKIN].attribute_good = 0xff;
  g_characters[MORKIN].attribute_bad = 0;
  g_characters[MORKIN].liege = 0;
  g_characters[MORKIN].flags &= 0xf8; //loyalty = 0
  
}

// recruit them , recruited by us
void recruitSomeone(int them, int us) {
  g_characters[them].time_looking = MASK_LOOKING(g_characters[us].time_looking) |
                                    (g_characters[them].time_looking&7);
  
  g_characters[them].liege = us;
  g_characters[them].flags = (g_characters[them].flags&0xf8) | (LOYALTY(g_characters[us].flags));
  
  // if their foe is loyal to the same person as us...
  if (LOYALTY(g_characters[g_characters[them].foe].flags) == 
      LOYALTY(g_characters[us].flags) ) 
  {
    g_characters[them].foe = g_characters[us].foe;
  }
  
}


int doApproach(int them, int us) {
  
  if (them == MORKIN) {
    if (us != TARITHEL) {
      return 0;
    } else {
      recruitMorkin();
      return 1;
    }
  }
  if (them <= SHARETH) {
    return 0;
  }
  
  /*Begin recruitment check. 
    If 4 or more of our good/bad attributes match, then recruit successfully
    NB, attributes swift / slow are not counted
    Small bonus for recruiting non-loyal characters
    bonus of current rate * 2 for recruiting treacherous characters
    Big bonus for recruiting characters who have us as our liege
    Medium bonus for recruiting when carrying object 0-4
  */
  int approachKey = ( (g_characters[them].attribute_bad & g_characters[us].attribute_bad) |
                      (g_characters[them].attribute_good & g_characters[us].attribute_good) ) 
                      & 0xbf;
  int i;
  int success = 0;
  // loop over the 8 bits of approach key, increase chance each time there is a match
  for (i = 0; i < 8; i++) {
    if (approachKey & 1)
      success++;
    approachKey /= 2;
  }
  
  if (!IS_LOYAL( g_characters[them].attribute_good))
    success++;
    
  if (IS_TREACHEROUS(g_characters[them].attribute_bad))
    success *= 2;
  
  if (g_characters[them].liege == us) 
    success+=3;
  
  if ( g_characters[us].object < 5) 
    success+=2;
  
  if (success < 4) // failed
    return 0;
  // succeeded
  recruitSomeone(them,us);
  return 1;
}


void describeWhereCharIs(int them) {
  //77a7
  createCharacterTitle(them);
  addSingleToken(0x2d);
  addAtLocation(&g_characters[them].location);
  
}

void describeGuidance(void) {
  addToTokenList(dash_);
  
  int who = randomish()&127;
  if (LOYALTY(g_characters[who].flags) != 0 &&
      g_characters[who].reckless
  ) {
    describeWhereCharIs(who);
    addGuidanceLog(who);
  }
  else {
    // random was an ally or a dead person - so describe where our object is
    int ourObject = g_saveArea.selectedChar;
    if (g_characters[g_saveArea.selectedChar].object != ourObject) {
      // we are not carrying our object, tell us where it is
      addFullObjectDescription(g_saveArea.selectedChar); // crown of blah
      addToTokenList(canBeFound);   // can be found
      
      if (g_objects[ourObject].x == 0xff) {
        // someone has it!
        addToTokenList(inTheHandsOf);     
        int whoHasIt = g_objects[ourObject].y;
        createCharacterTitle(whoHasIt);     // rothrorn the wise
        addAtLocation(&g_characters[whoHasIt].location);  // on the plains of despair
        
      } else {
        addAtLocation(&g_objects[ourObject]);  // on the plains of despair
      }
      addGuidanceLog(ourObject | GUIDANCE_OBJECT);
    } else {
      // already have our object 
      // describe where baddy is
      describeWhereCharIs(SHARETH);
      addGuidanceLog(who);
    }
  }  
}


void describeObjectCarried(int who) {
  //785f  
  createCharacterName(who);
  addSingleToken(0x9a); //"carries"
  if (g_characters[who].object == 0xff) {
    addSingleToken(0x9b); //nothing
  } else {
    addFullObjectDescription(g_characters[who].object);
  }
  addLiteralToken('.');
  
}


void addPreparesToBattle(int who) {
  if (g_localDetails.unloyals_here) {
    createCharacterTitle(who);
    addToTokenList(preparesToBattle);
  }
  
}

void addCharAndKiller(int who) {
  createCharacterTitle(who);
  addToTokenList(isDeadSlainBy);
  if (g_characters[who].object < 0x80) {
    createCharacterTitle(g_characters[who].object);
  }
  else {
    addSingleToken(g_characters[who].object-5);
  }
  addLiteralToken('.');
  
}


void checkDeadChar(int who) {
  if (g_characters[who].reckless)
    return;
  
  //66d6
  addNewlineToken();
  addCharAndKiller(who);
  print_tokens_pal(FONT_GREEN_B);
  
}

// 754b
// 51 is liege, 52 foe
void createLiegeFoe(int liegeOrFoe, int who) {
  
  int parent;
  if (liegeOrFoe == 0x51) {
    parent = g_characters[who].liege;
  }
  else {
    parent = g_characters[who].foe;
  }
  
  if (parent >= 0x80) // is not used
    return;
  
  createCharacterName_s(who); // "Someone's"
  addSingleToken(liegeOrFoe); // "liege"/"foe"
  addSingleToken(0x34); //"is"
  createCharacterTitle(parent); // "fullname the whatever"
  addLiteralToken('.');
  
}

  

// 74cf
void describeCharacter(int who) {
  int i, needComma, attributeToken;
  
  createCharacterTitle(who);
  addSingleToken(0x34); //"is"
  
  u16 attributes = g_characters[who].attribute_good | (g_characters[who].attribute_bad<<8);
  needComma = 0;
  attributeToken = 0x34;
  // loop over 16 bits of attribute data, add attribute whenever lowest bit is set
  for (i = 0; i < 16; i++) {
    attributeToken++;
    if (attributes&1) {
      // have this attribute
      if (needComma) {
        addLiteralToken(',');
      }
      addSingleToken(attributeToken);
      needComma = 1;
    }
    attributes>>=1;
  }
  
  addSingleToken(0x50); //"and"
  addUtterly(g_characters[who].reckless); //"utterly"
  addToTokenList(reckless_);  //"reckless."
  
  createCharacterName(who); //"Luxor"
  addSingleToken(0x34); //"is"
  addUtterly(g_characters[who].despondence); //"utterly"
  addToTokenList(despondentAndIs);  //"despondent and is"
  addUtterly(g_characters[who].energy); //"utterly"
  addToTokenList(tired_); //"tired."
  
  createLiegeFoe(0x51, who);
  createLiegeFoe(0x52, who);
  
}


