/*
*  $Log$
*  Revision 2.4  2004/02/22 15:17:55  richq
*  overflow bug in recruitment fixed
*
*  Revision 2.3  2004/02/18 20:09:48  richq
*  Bug fix - posted army type displayed wrong
*
*  Revision 2.2  2004/02/18 19:24:34  richq
*  Creates the list of actions we can take in a given situation. Then handles the
*  user selection of those actions.
*
*  Revision 2.1.1.1  2004/02/04 21:49:21  Default
*  Doomdark's Revenge GBA
*
*
*/

#include "choose.h"
#include "select.h"
#include "objects.h"
#include "tokens.h"
#include "tokendata.h"
#include "savearea.h"
#include "army.h"
#include "features.h"
#include "characters.h"
#include "infoscreen.h"
#include "main.h"
#include "gfx.h"
#include "menu.h"
#include "look.h"
#include "blit.h"

typedef struct {
s8 give;                    // 5df6     needed value = !0xff
s8 take;                    // 5df5     needed value = !0xff
u8 enter;                   // 5df4     needed value = !0x00
u8 recruit;                 // 5df9     needed value = !0x00
u8 post;                    // 5dfa     needed value = !0x00
u8 attack;//unloyals ahead  // 5c13     needed value = !0x00
s8 approach;                // 5df7     needed value = !0xff
s8 use;                     // 5df8     needed value = !0xff
} choices_t;

// global choices...
choices_t g_choices;


void rest(void);
void take(void);
void give(void);
void enter(void);
void recruit(void);
void post(void);
void attack(void);
void approach(void);
void use(void);

// jump table
void (*const choiceTable[]) (void) = {
  rest,
  take,
  give,
  enter,
  recruit,
  post,
  attack,
  approach,
  use
};


// which choice is selected
int g_hiliteChoice;

void addChoice(int choice) {
//  addNewlineToken();
//  addLiteralToken(choice);
  addLiteralToken('#');
  addUppercaseToken();
  addSingleToken(choice + 0x7e);  //; add 7e  ('1' = 31, 7e+31 = af == "rest")
  g_selectableThingTable[g_selectableThings++] = (u8)choice;
  print_tokens_pal(FONT_RED);
  g_indent = 4;


}




// initialise the choice flags - 7bb2
void initChoices(void) {

  // work out what is here
  // call 7a12
  getEntranceAndObject(&g_characters[g_saveArea.selectedChar].location, 
      &g_choices.take,
      &g_choices.enter);

  g_choices.give = -1;
  g_choices.approach = -1;
  g_choices.use = -1;
  g_choices.recruit = 0;
  g_choices.post = 0;

  // call 7092 ; get char and armies here
  getCharactersAt(&g_characters[g_saveArea.selectedChar].location);
  if (g_total_here == 2 &&
      g_characters[g_saveArea.selectedChar].object != 0xff &&
      IS_GENEROUS(g_characters[g_saveArea.selectedChar].attribute_good) )
  {
    // can we give our object away?
    int them = g_charactersHere[0];
    if (them == g_saveArea.selectedChar)
      them = g_charactersHere[1];

    if (g_characters[them].object != 0xff) {
      g_choices.give = them;
    }
  }


  g_choices.attack = g_localDetails.unloyals_infront;
  if (IS_COWARDLY(g_characters[g_saveArea.selectedChar].attribute_bad) ) {
    // cowards don't fight
    g_choices.attack = 0;
  }

  if (g_localDetails.garrison_here) {
    // an army is at our current location
    if (!getArmyLoyalty(g_localDetails.garrison_here) && /*Loyal to moon prince*/
        getArmyRace(g_localDetails.garrison_here)
              == getArmyRace(g_saveArea.selectedChar)        /*No mixed race armies allowed*/
        )
    {
      // check if we can recruit:
      if (getArmySize(g_localDetails.garrison_here) > 30 && /*they have > 150 soldiers*/
          getArmySize(g_saveArea.selectedChar) < 0xe7)  /* we have < 1155 soldiers*/
      { 
        g_choices.recruit = g_localDetails.garrison_here;
      }
      // check if we can post
      if (getArmySize(g_localDetails.garrison_here) < 0xe7 && /* they have <= 1150 soldiers*/
          getArmySize(g_saveArea.selectedChar) > 30  )            /* we have > 150 soldiers*/
      {
        g_choices.post = g_localDetails.garrison_here;
      }
    }
  } /* End garrison checks */

  if ( !USED_OBJECT(g_characters[g_saveArea.selectedChar].flags) &&
        g_characters[g_saveArea.selectedChar].object == g_saveArea.selectedChar)
  {
      g_choices.use = g_saveArea.selectedChar;
  }/* End object use checks */

  getCharactersAt(&g_inFront);
  if (g_total_here == 1 &&
      LOYALTY(g_characters[g_charactersHere[0]].flags) != 0
  ) {
    g_choices.approach = g_charactersHere[0];
  }


}


void createChoiceList(void) {
  addChoice(0x31);
  print_tokens_newline_0indent(FONT_RED);

  if (g_choices.take != -1) {
    addChoice(0x32);
    // describe object
    addShortObjectDescription(g_choices.take);
    print_tokens_newline_0indent(FONT_RED);
  }

  if (g_choices.give != -1) {
    addChoice(0x33);    // "give"
    // describe object carried
    addShortObjectDescription(g_characters[g_saveArea.selectedChar].object); //"the sword of blah"
    addSingleToken(0x2c); //"to"
    createCharacterTitle(g_choices.give );
    print_tokens_newline_0indent(FONT_RED);
  }

  if (g_choices.enter != 0) {
    addChoice(0x34);    // "enter"
    addToTokenList(theTunnel);
    print_tokens_newline_0indent(FONT_RED);
  }

  if (g_choices.recruit != 0) {
    addChoice(0x35);    // "recruit"
    addArmyTypeTokens(g_choices.recruit);
    print_tokens_newline_0indent(FONT_RED);
  }

  if (g_choices.post != 0) {
    addChoice(0x36);    // "post"
    addArmyTypeTokens(g_choices.post);
    print_tokens_newline_0indent(FONT_RED);
  }

  if (g_choices.attack != 0) {
    addChoice(0x37);    // "attack"
    addToTokenList(theFoe);
    print_tokens_newline_0indent(FONT_RED);
  }

  if (g_choices.approach != -1) {
    addChoice(0x38); //aproach
    createCharacterTitle(g_choices.approach); //bob the builder
    print_tokens_newline_0indent(FONT_RED);
  }
  if (g_choices.use != -1) {
    addChoice(0x39); //aproach
    addShortObjectDescription(g_choices.use);
    print_tokens_pal(FONT_RED);
  }
  // call create choices again... not needed as g_choices is not overwritten!
}


// use g_selectableThings and g_selectableThingTable[32] to
// keep track of what we can do
// createChoices taken from 7ab7 this time
void createChoices(void) {

  drawFeatureAndChar();
  if ( TIME_LEFT(g_characters[g_saveArea.selectedChar].time_looking) == 0) {
    // It is night. Luxor the Moonprince can do nothing
    addUppercaseToken();
    addTimeTokens(TIME_LEFT(g_characters[g_saveArea.selectedChar].time_looking));
    createCharacterTitle(g_saveArea.selectedChar);
    addToTokenList(canDoNothing);
    print_tokens_pal(FONT_RED);
  } else {
    // 7ad4
    // CALL 7bb2 ; check everything that can occur
    initChoices();
    createChoiceList();

  }


}

// this time, is at 847f
void choose(void) {
  g_status.screen = CHOOSE_SCREEN;
  if (checkLocationAndStatus())
    return;
  g_selectableThings = 0;
  createChoices();
  g_hiliteChoice = 0;
  selectOption(g_hiliteChoice);

}


void makeChoice(void) {
  if ( TIME_LEFT(g_characters[g_saveArea.selectedChar].time_looking) == 0)
    return;
    
  int choice = g_selectableThingTable[g_hiliteChoice]-'1';
  clearHighlighter();
  choiceTable[choice]();

}

void chooseDown(void){
  if ( TIME_LEFT(g_characters[g_saveArea.selectedChar].time_looking) == 0)
    return;
    
  g_hiliteChoice++;
  if (g_hiliteChoice < g_selectableThings) {
    selectOption(g_hiliteChoice);

  } else {
    g_hiliteChoice--;
  }

}

void chooseUp(void){
  if ( TIME_LEFT(g_characters[g_saveArea.selectedChar].time_looking) == 0)
    return;
  if (g_hiliteChoice) {
    g_hiliteChoice--;
    selectOption(g_hiliteChoice);
  }
}

void rest(void) {     //8681
  int tl = TIME_LEFT(g_characters[g_saveArea.selectedChar].time_looking);
  increaseU8By(&g_characters[g_saveArea.selectedChar].energy, tl);
  g_characters[g_saveArea.selectedChar].time_looking &= 7;
  
  look();
}
void take(void) {     //8695
  if (g_choices.take != -1) {
    character_t * selectedChar = &g_characters[g_saveArea.selectedChar];
    if (selectedChar->object != 0xff) {
      g_objects[selectedChar->object].x = selectedChar->location.x;
      g_objects[selectedChar->object].y = selectedChar->location.y;
    }
    selectedChar->object = g_choices.take;
    g_objects[selectedChar->object].x = 0xff;
    g_objects[selectedChar->object].y = g_saveArea.selectedChar;
    
    infoPlace(1);
  }
}
void give(void) {     //86ca
  if (g_choices.give != -1) {
    character_t * selectedChar = &g_characters[g_saveArea.selectedChar];
    g_characters[g_choices.give].object = selectedChar->object;
    g_objects[selectedChar->object].x = 0xff;
    g_objects[selectedChar->object].y = g_choices.give;
    selectedChar->object = 0xff;
    infoPlace(1);
  }
}
void enter(void) {    //86f4
  if (g_choices.enter != 0) {
    clearBuffer();
    blit_buffer();
    character_t * selectedChar = &g_characters[g_saveArea.selectedChar];
    selectedChar->flags |= 0x40; //bit 6
    int time = MASK_LOOKING(selectedChar->time_looking);
    selectedChar->time_looking = 0;
    
    // look around until we find the direction the tunnel entrance goes:
    for (;;) {
      checkVision(g_saveArea.selectedChar);
      if ( HAS_TUNNEL(g_locdat.info) ) {
        clearArmyTiles();
        selectedChar->time_looking |= time;
        selectedChar->location.x = g_inFront.x;
        selectedChar->location.y = g_inFront.y;
        setArmyTiles();
        look();
        return;
      }
      selectedChar->time_looking += 2;
    }
  }
  
}

void recruit(void) {  //871a
  if (g_choices.recruit != 0) {
    u8 size = getArmySize(g_choices.recruit);
    decreaseU8By(&size,20);
    setArmySize(g_choices.recruit, size);
    
    size = getArmySize(g_saveArea.selectedChar);
    increaseU8By(&size,20);
    setArmySize(g_saveArea.selectedChar, size);
    
    infoLocation();
    
  }
}

void post(void) {     //8738
  if (g_choices.post != 0) {
    u8 size = getArmySize(g_choices.post);
    increaseU8By(&size,20);
    setArmySize(g_choices.post, size);
    
    size = getArmySize(g_saveArea.selectedChar);
    decreaseU8By(&size,20);
    setArmySize(g_saveArea.selectedChar, size);
    
    infoLocation();
  }
  
}

void attack(void) {   //8756
  if (g_choices.attack != 0) {
    // set to night:
    g_characters[g_saveArea.selectedChar].time_looking &= 7;
    
    clearArmyTiles();
    g_characters[g_saveArea.selectedChar].location.x = g_inFront.x;
    g_characters[g_saveArea.selectedChar].location.y = g_inFront.y;
    setArmyTiles();
    look();
  }
}

void approach(void) { //8777
  if (g_choices.approach != -1) {
    checkVision(g_saveArea.selectedChar);
    doMove();
    int success = doApproach(g_choices.approach, g_saveArea.selectedChar);
    if (!success) {
      g_characters[g_saveArea.selectedChar].time_looking &= 7;
    } else {
      g_saveArea.selectedChar = g_choices.approach;
      //setSelectedChar(g_choices.approach);
    }
    clearHighlighter();
    look();
  }
  
}

void doUsageEffect(int object) {
  // 67ab
  if (object < 5) {
    g_characters[g_saveArea.selectedChar].energy = 255;
    g_characters[g_saveArea.selectedChar].despondence = 255;
  }
  else {
    // a "special" item
    if (object != 2 && object != 3) {
      // bring all loyal chars to us
      clearArmyTiles();
      int i;
      for (i = 0; i < 128; i++) {
        if (LOYALTY(g_characters[i].flags) == 0 &&
          g_characters[i].reckless ) 
        {
          g_characters[i].location = g_characters[g_saveArea.selectedChar].location;
          if (g_status.charInTunnel) {
            g_characters[i].flags |= IN_TUNNEL_FLAG;
          } else {
            g_characters[i].flags &= ~IN_TUNNEL_FLAG;
          }
        }
      }
      setArmyTiles();
    } else if (object == 2) {
      // take us to morkin
      g_characters[g_saveArea.selectedChar].location = g_characters[1].location;
      // sort out tunnel flag:
      u8 flags = g_characters[g_saveArea.selectedChar].flags;
      flags = (flags&~IN_TUNNEL_FLAG)|(g_characters[1].flags&IN_TUNNEL_FLAG);
      g_characters[g_saveArea.selectedChar].flags = flags;
      
    } else if (object == 3) {
      // refresh all loyal chars
      int i;
      for (i = 0; i < 128; i++) {
        if (LOYALTY(g_characters[i].flags) == 0) 
        {
          g_characters[i].energy = 255;
          g_characters[i].despondence = 255;
        }
      }
    }
  }
}


void use(void) {      //879e
  if (g_choices.use != -1) {
    g_characters[g_saveArea.selectedChar].flags |= USED_OBJECT_FLAG; // object used
    doUsageEffect(g_choices.use);
    infoPlace(1);
  }
}

