#include "stdlib.h"
#include "gba.h"
#include "screenmode.h"
#include "keypad.h"

#include "characters.h"
#include "gfx.h"
#include "infoscreen.h"
#include "tokens.h"
#include "tokendata.h"
#include "guidance.h"
#include "savearea.h"
#include "menu.h"

#define LOG_SIZE 5

int g_showing;
int g_historyDepth;
guidance_t * g_guidanceLog;

void init_guidance(void) {
  if (g_guidanceLog == 0) {
    g_guidanceLog = (guidance_t * )malloc(LOG_SIZE*sizeof(guidance_t));
  }
  g_historyDepth = 0;
}

const u8 noGuidance[] = {0xfc, 0x55, 0x80, 0x9c, 0xa1, // No guidance has be
                        0xfe, 'e', 0xfe, 'n', 0x9d, 0xff}; //en found.

// shows a simple guidance/log screen
void showLogEntry(void) {
  
  if (g_showing >= g_historyDepth)
    g_showing = 0;
  
  if (g_historyDepth != 0) {
    
    g_saveArea.selectedChar = g_guidanceLog[g_showing].receiver;
    drawFeatureAndChar();
    createCharacterTitle(g_saveArea.selectedChar);
    addToTokenList(hasFound);
    addSingleToken(0x80); // guidance
    addToTokenList(dash_);
//    addUppercaseToken();
    if (g_guidanceLog[g_showing].whoOrWhat >= GUIDANCE_OBJECT) {
      addFullObjectDescription(g_guidanceLog[g_showing].whoOrWhat&0x7f); // crown of blah
      addToTokenList(canBeFound);   // can be found
      
      if (g_guidanceLog[g_showing].location.x == 0xff) {
        // someone has it!
        addToTokenList(inTheHandsOf);     
        int whoHasIt = g_guidanceLog[g_showing].location.y;
        createCharacterTitle(whoHasIt);     // rothrorn the wise
        addAtLocation(&g_guidanceLog[g_showing].ownerLocation);  // on the plains of despair
        
      } else {
        addAtLocation(&g_guidanceLog[g_showing].location);  // on the plains of despair
      }
      
    } else {
      createCharacterTitle(g_guidanceLog[g_showing].whoOrWhat);
      addSingleToken(0x2d); // stands
      addAtLocation(&g_guidanceLog[g_showing].location);
    }
    addLiteralToken('.');
    if (g_showing < g_historyDepth-1) {
      addToTokenList(_More);
    }
  }
  else {
    drawFeatureAndChar();
    addToTokenList(noGuidance);
  }
  
  print_tokens_pal(FONT_RED);
  
}


// keep polling key presses until B is pressed
void handleLogKeys(void) {
  int delay = 2;
  while(1) {
    UpdateKeys();
    wait_vsync_int();
    if (KeyPressedNoBounce(KEY_B,delay)) {
      break;
    }
    
    if (KeyPressedNoBounce(KEY_A,delay)) {
      g_showing++;
      showLogEntry();
    }
  }
}

void showGuidanceLog(void) {
  int selectedChar = g_saveArea.selectedChar;
  g_showing = 0;
  clearHighlighter();
  showLogEntry();
  handleLogKeys();
  g_saveArea.selectedChar = selectedChar;
  setupInfoScreen();
}

// lose the last entry...
void pushGuidanceStack(void) {
  
  int i;
  for (i = LOG_SIZE-1; i > 0; i--) {
    g_guidanceLog[i] = g_guidanceLog[i-1];
  }
  
}


// caches the guidance - stores current char or object location 
// and, if someone is carrying our object, their location
void addGuidanceLog(int whoOrWhat) {
  
  // shuffle the history stack so entry 0 is unused
  pushGuidanceStack();
  
  g_historyDepth++;
  
  g_guidanceLog[0].whoOrWhat = whoOrWhat;
  g_guidanceLog[0].receiver = g_saveArea.selectedChar;
  
  if (whoOrWhat >= GUIDANCE_OBJECT) {
    int what = whoOrWhat&0x7f;
    // store object location as is right now
    g_guidanceLog[0].location = g_objects[what];
    if (g_objects[what].x == 0xff) {
      int whoHasIt = g_objects[what].y;
      // store owner location as is right now
      g_guidanceLog[0].ownerLocation = g_characters[whoHasIt].location;
    }
  } else {
    g_guidanceLog[0].location = g_characters[whoOrWhat].location;;
  }
  
}

