/*
* $Log$
* Revision 2.2  2004/02/22 15:31:43  richq
* Turn off level bg between screens to try and fix transition porblems.
* Added 'none' when army.slew is 0.
*
* Revision 2.1  2004/02/18 19:10:02  richq
* Contains the information screens, those that were originally called with R,T,Y,U
* Routines for More?, battle reports, and drawing the current feature with char.
* Also has the game over screen and the screen displayed when a char is killed.
*
*/

#include "keypad.h"
#include "screenmode.h"

#include "infoscreen.h"
#include "characters.h"
#include "objects.h"
#include "gfx.h"
#include "blit.h"
#include "tokens.h"
#include "tokendata.h"
#include "army.h"
#include "main.h"
#include "savearea.h"

#define INFOSCRN_CHAR_X_POS     16
#define INFOSCRN_CHAR_Y_POS     160
// really this is position in buffer:
#define INFOSCRN_FEATURE_X_POS  0x160
#define INFOSCRN_FEATURE_Y_POS  40

// the flag for if more and where we are moring (in front or current location)
int g_hasMore;
// the more to describe
int g_whoToDescribe;

void moreInFront(void);
void moreHere(void);

/*
clears the screen, the text, removes any objects
sets the palette background to grey/white
*/
void setupInfoScreen(void) {
  // clear top screen
  wait_vsync_int();
  REG_DISPCNT &= ~BG0_ENABLE;
  clear_view();
  // clear bottom
  clear_objects();
  // clear shield...
  remove_shield();

  CopyOAM();
  set_think_palette();
  set_object_palette(/*0*/);
  clear_text();

  clearBuffer();
  blit_buffer();
  //setup_view();

  set_print_vars(0, 0, 0, 0x1e);
  
}


/*
Draws the blank screen with the character and feature
character drawn is g_saveArea.selectedChar,
feature is whatever feature they are stood on
7593
*/
void drawFeatureAndChar(void) {
  // white out
  setupInfoScreen();
  character_t * selectedChar = &g_characters[g_saveArea.selectedChar];
  if (!IN_TUNNEL(selectedChar->flags)) {
    // calculate where we are
    g_locdat.x = selectedChar->location.x;
    g_locdat.y = selectedChar->location.y;
    fillLocationData(&g_locdat);
    // draw the feature at size 0, bottom right of screen
    drawFeature(g_locdat.terrain, 0,
                INFOSCRN_FEATURE_X_POS, INFOSCRN_FEATURE_Y_POS);

    blit_buffer();
    setup_view();
    bg_level.y_scroll = 256-(160-80);
    UpdateBackground(&bg_level);

  }
  // get char race, use as gfx and draw with "palette change type 2"
  drawCharGfx(selectedChar->race_orders&0xf,
              INFOSCRN_CHAR_X_POS,
              INFOSCRN_CHAR_Y_POS,
              COLOURTYPE_INFO);
  createCharacterTitle(g_saveArea.selectedChar);
  addNewlineToken();
  addNewlineToken();
  print_tokens(0, 0, 0, 0x1e, FONT_BLUE);

}


//7a5f describe entrance and objects here
// e.g. "Luxor the Moonprince sees the sword of Thigrim whose
//        power is in battle and an underground entrance."
void describeEntranceAndObject(void) {
  s8 take;
  u8 enter;
  getEntranceAndObject(&g_characters[g_saveArea.selectedChar].location,
      &take,
      &enter);
  if ( (take + enter) == -1)
    return;

  createCharacterName(g_saveArea.selectedChar);  //"Luxor the Moonprince"
  addSingleToken(0xab); // "sees"

  if (take != -1) {
    addFullObjectDescription( take );
    if (enter != 0)
      addSingleToken(0x50); // "and"
  }

  if (enter != 0)
    addToTokenList(anUndergroundEntrance);

  addLiteralToken('.');

}


// checks that there are really more chars here
// makes g_total_here-1 index the next character
void moreHere(void) {
  // 85d7
  while (g_charactersHere[g_total_here-1] == g_saveArea.selectedChar) {
    g_total_here--;

    if (g_total_here == 0) {
      moreInFront();
      return;
    }
  }

  addMoreAndPrint();

}


//856e
// checks if there are characters in front, then when we are
// finished infront too, sets the check flag to 0
void moreInFront(void) {
  if (g_hasMore == 1) {
    g_hasMore = 0;
  } else {
    g_hasMore = 1;
    getCharactersAt(&g_inFront);
    if (g_total_here == 0) {
      g_hasMore = 0;
    } else {
      //85d7
      moreHere();
    }
  }
}


// check if we need to write "more?"
// 858e
void restOfInfo(void) {
  g_hasMore = 2;
  getCharactersAt(&g_characters[g_saveArea.selectedChar].location);
  if (g_total_here < 2) {
    moreInFront();
  } else {
    //85d7
    moreHere();
  }

}


// show info on a place. 84bc
void infoPlace(int doStatCheck) {


  if (doStatCheck) {
    if (checkLocationAndStatus())
      return;
  }

  g_status.screen = PLACE_SCREEN;
  drawFeatureAndChar();
  addTimeTokens(TIME_LEFT(g_characters[g_saveArea.selectedChar].time_looking));

  describeEntranceAndObject();

  addToTokenList(Here_);
  if (g_status.charInTunnel) {
    addSingleToken(0x2a); // "in"
    addToTokenList(theTunnel);
  }
  else {
    addAtLocation(&g_characters[g_saveArea.selectedChar].location);
  }
  addLiteralToken(',');
  createCharacterTitle(g_saveArea.selectedChar);
  addToTokenList(hasFound);
  describeObjects();

  addLiteralToken('.');
  describeObjectCarried(g_saveArea.selectedChar);
  addPreparesToBattle(g_saveArea.selectedChar);
  print_tokens_pal(FONT_RED);

  g_locdat.x = g_characters[g_saveArea.selectedChar].location.x;
  g_locdat.y = g_characters[g_saveArea.selectedChar].location.y;
  fillLocationData(&g_locdat);

  // add more etc
  restOfInfo();
}

void lordSlew(void) {
  //7965
  createCharacterName(g_saveArea.selectedChar);  //"Luxor"
  addSingleToken(0xa9); //"slew"
  if ( KILLED_FOE(g_characters[g_saveArea.selectedChar].flags)) {
    createCharacterTitle(g_characters[g_saveArea.selectedChar].fighting);
  }
  else {
    addToTokenList(noOne);
  }
  addLiteralToken('.');

}


// create battle report for current char
void createBattleStatReport(void) {
  // 78b1
  if ( !IN_BATTLE(g_characters[g_saveArea.selectedChar].flags)) {
    createCharacterTitle(g_saveArea.selectedChar);
    addToTokenList(foughtNothing);
    return;
  }

  addToTokenList(inBattle);
  if (g_characters[g_saveArea.selectedChar].fighting < 0x80) {
    addSingleToken(0xd8);  //"against"
    createCharacterTitle(g_characters[g_saveArea.selectedChar].fighting);
  }
  addLiteralToken(',');
  createCharacterTitle(g_saveArea.selectedChar);
  addSingleToken(0xa6); //"won"

  if ( WON_BATTLE(g_characters[g_saveArea.selectedChar].flags)) {
    addSingleToken(0x32); //"a"
  }
  else {
    addSingleToken(0x55); //"no"
  }

  addToTokenList(victory_);

  // now add the losses, etc
  if ( (g_charArmies[g_saveArea.selectedChar].lost |
        g_charArmies[g_saveArea.selectedChar].size) == 0 ) {
    // just do the "lord slew" string
    lordSlew();
  } else {
    createCharacterName(g_saveArea.selectedChar);  //"Luxor"
    addSingleToken(0xa7); //"lost"
    numberToWords(g_charArmies[g_saveArea.selectedChar].lost*5); // "ten"
    addArmyTypeTokens(g_charArmies[g_saveArea.selectedChar].race); // "riders"
    addLiteralToken('.');
    // print who lord slew
    //CALL 7965
    lordSlew();
    createCharacterName_s(g_saveArea.selectedChar); //"Luxor's"
    addArmyTypeTokens(g_charArmies[g_saveArea.selectedChar].race); // "riders"
    addSingleToken(0xa9); //"slew"
    if (g_charArmies[g_saveArea.selectedChar].slew) {
      numberToWords(g_charArmies[g_saveArea.selectedChar].slew*5); // "one hundred"
    }
    else {
      addSingleToken(0x72); //"none"
    }
    addToTokenList(ofTheFoe);//"of the foe."
  }

}


/*Current character battle status: T pressed*/
void battleStats(void) {
  g_status.screen = BATTLE_STATS_SCREEN;
  if (checkLocationAndStatus())
    return;

  drawFeatureAndChar();
  createBattleStatReport();
  print_tokens_pal(FONT_RED);

}


/* 8544
  ; Y pressed (check location)
*/
void infoLocation(void) {
  g_status.screen = LOCATION_SCREEN;
  if (checkLocationAndStatus())
    return;
  drawFeatureAndChar();
  createCharArmyReport(g_saveArea.selectedChar);
  print_tokens_pal(FONT_RED);

  if (!g_status.charInTunnel) {
    // rest of info, if we are at a fortress or in front of one
    describeFortress(&g_characters[g_saveArea.selectedChar].location,
                     0xfc, //caps
                     g_localDetails.garrison_here);
    describeFortress(&g_inFront,
                     0x79, // "ahead"
                     g_localDetails.garrison_infront);
  }

  restOfInfo();
}


void initMoreScreen(void) {
  // 85fb
  drawFeatureAndChar();
  addUppercaseToken();
  location_t * where;
  int whereToken;
  if (g_hasMore == 2) {
    // searching our current loc
    whereToken = 0x78; // "here"
    where = &g_characters[g_saveArea.selectedChar].location;
  } else {
    // searching ahead
    whereToken = 0x79; // "ahead"
    where = &g_inFront;
  }

  addSingleToken(whereToken);

  if (g_status.charInTunnel == 0)
    addAtLocation(where);

  addSingleToken(0x2d); // stands

  addTitleAndLoyalty(g_whoToDescribe);

}


void moreLocationsScreen(void) {
  // 8633
  initMoreScreen();
  createCharArmyReport(g_whoToDescribe);
  print_tokens_pal(FONT_RED);

}


void morePlaceScreen(void) {
  //863c
  initMoreScreen();
  if (LOYALTY(g_characters[g_whoToDescribe].flags) ) {
    // only describe our enemies
    // 74cf
    describeCharacter(g_whoToDescribe);
  }

  if (g_characters[g_whoToDescribe].object != 0xff)
    describeObjectCarried(g_whoToDescribe);

  print_tokens_pal(FONT_RED);
}

/*
  85a0
  called on "a" press when place/location screen and has more
*/
void showMoreInfo(void) {
  if (g_hasMore == 0)
    return;

  g_whoToDescribe = g_charactersHere[g_total_here-1];

  if (g_status.screen == LOCATION_SCREEN) {
    // 8633
    moreLocationsScreen();
  }
  else {
    // PLACE_SCREEN
    // 863c
    morePlaceScreen();
  }

  // 85cb
  g_total_here--;
  if (g_total_here == 0) {
    moreInFront();
    return;
  } else {
    moreHere();
  }


}

// U pressed
void infoCharacter(void) {
  g_status.screen = CHARACTER_SCREEN;
  if (checkLocationAndStatus())
    return;

  drawFeatureAndChar();
  describeCharacter(g_saveArea.selectedChar);
  print_tokens_pal(FONT_RED);

}

void lostGame(void) {
  //setSelectedChar(4);
  g_saveArea.selectedChar = SHARETH;
  drawFeatureAndChar();
  addCharAndKiller(LUXOR);
  addToTokenList(evilSharethWins);
  g_gameover = 1;
  print_tokens_pal(FONT_RED);

  WaitKeyPress(KEY_A);
  WaitKeyReleased(KEY_A);
  
}

void deadScreen(void) {  
  g_status.screen = DEAD_SCREEN;
  drawFeatureAndChar();
  addCharAndKiller(g_saveArea.selectedChar);

  if ( IN_BATTLE(g_characters[g_saveArea.selectedChar].flags)) {
    createBattleStatReport();
  }
  print_tokens_pal(FONT_RED);
}

