/*
* $Log$
* Revision 2.3  2004/02/22 15:02:35  richq
* Added the saving of the guidance log and the map we've seen.
*
* Revision 2.2  2004/02/18 19:13:55  richq
* Similar to LoM, with changes reflecting the changes in data structures.
* Really this is a "pause menu", with save, load and options. Access to
* a map added in DR.
*
* Revision 2.1.1.1  2004/02/04 21:49:21  Default
* Doomdark's Revenge GBA
*
*
*/

#include "screenmode.h"
#include "stdlib.h"
#include "keypad.h"
#include "saving.h"

#include "savearea.h"
#include "look.h"
#include "tokens.h"
#include "tokendata.h"
#include "gfx.h"
#include "menu.h"
#include "location.h"
#include "characters.h"
#include "infoscreen.h"
#include "main.h"
#include "army.h"
#include "mapscreen.h"
#include "guidance.h"

savearea_t g_saveArea;
int l_highlightSlot;
int l_maxHighlight;

void init_savearea(void) {
//  if (g_saveArea == 0)
//    g_saveArea = (savearea_t *) malloc(sizeof(savearea_t));
  g_saveArea.days = 0;
  g_saveArea.selectedChar = 0;

}

// individual size of save data (is slightly bigger than necessary):
//#define SAVE_SIZE 8840
//#define SAVE_SIZE 8964
//#define SAVE_SIZE 8976 
//#define SAVE_SIZE 10496
//#define SAVE_SIZE 10500
#define SAVE_SIZE 10530
  // was 5400
// slots
#define SAVE_SLOTS 3
#define SAVE_OPTIONS 7
// skip first few bytes, as they are dodgy (?)
#define SAVE_OFFSET 3
// header position is offset + 3 byte "key"
#define SAVE_HEADER_POS (SAVE_OFFSET+3)
// the key to look for - so we know this is our save data
const char * const saveKey = "DDR";

// header. Add to this later
#define HEADER_SIZE 1
typedef struct {
  u8 isSaved;   // bit flag for which slots are filled
  // time played? this would have to be per game
//  u16 hours;
//  u8 minutes;

}
header_t;

// start of save data
#define SAVE_DATA_POS (SAVE_HEADER_POS + HEADER_SIZE)

const u8 gameSaved[] = {0xfc,0xdc,0xfc,0xd9,0xfe,0x64,0xfe,0x21,0xff };
const u8 saveGame[] = {0xfc,0xd9,0xfc,0xdc,0xff };
const u8 loadGame[] = {0xfc,0xda,0xfc,0xdc,0xff };

const u8 deleteGame[] = {0xfc,0xdb ,0xfc,0xdc,0xff};
const u8 gameDeleted[] = {0xfc,0xdc,0xfc,0xdb ,0xfe,0x64,0xfe,0x21,0xff};
const u8 smoothLooking[] = {0xfc, 0xdd, 0xff};

const u8 guidanceFound[] = {0xfc, 0x80, 0x9d, 0xff}; 

/*
Saving: data needed:

days
current char

location_t g_objects
// g_fortresses don't change
garrison_t g_garrisons
chararmy_t g_charArmies
character_t g_characters (all values)
u8 g_map

*/


// is our save key in sram?
int hasSaveKey(void) {
  char key;
  int i;
  for (i = 0; i < 3; i++) {
    key = LoadByte(SAVE_OFFSET + i);
    if (key != saveKey[i])
      return 0;
  }
  return 1;
}


int loadGameDays(int gameid) {
  int offset = SAVE_DATA_POS + SAVE_SIZE*gameid;

  int days = LoadByte(offset++);
  days += 256*LoadByte(offset++);
  return days;
}


void saveHeader(const header_t * h) {
  int i;
  for (i = 0; i < 3; i++)
    SaveByte(SAVE_OFFSET + i, saveKey[i]);

  SaveByte(SAVE_HEADER_POS, h->isSaved);
}

void loadHeader(header_t * h) {
  int i = 0;
  h->isSaved = LoadByte(SAVE_HEADER_POS + i);
  i++;
}

// []1 Day one
// []2 None
//  .. etc ..
void addSlotEntry(int days, int id) {
  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('1'+id);
  if (days >= 0) {
    addNameToken(0x6a); // Day
    numberToWords(days+1);  // one, two, etc
  } else {
    addNameToken(0x72); // None
  }
}


// shows list of saves already present
void showCurrentList(void) {
  int i;
  addNewlineToken();
  // load the list of saves
  header_t header;
  if (!hasSaveKey()) {
    // no saved games, create save game header
    header.isSaved = 0;
    saveHeader(&header);
  }

  // OK, we have a header, now see which games are here...
  loadHeader(&header);
  u8 bitmask;
  int days;
  for (i = 0; i < SAVE_SLOTS; i++) {
    bitmask = i==0?1:1<<i;
    days = -1;
    if (header.isSaved & bitmask) {
      // add a save game name
      days = loadGameDays(i);
    }
    addSlotEntry(days, i);  // Day one, etc
  }

}

void doSavedInfoScreen(void) {
  clear_text();
  print_tokens(2,2,0,0x1e,FONT_RED);
  showCurrentList();
  l_maxHighlight = SAVE_SLOTS;
  print_tokens_pal(FONT_RED);
}

int (*actionPerformed)(void);
int (*actionCancelled)(void);

// save data in slot l_highlightSlot
int saveData(void) {
  u8 theByte;
  int i;
  header_t header;
  loadHeader(&header);
  header.isSaved |= (l_highlightSlot==0)?1:(1<<l_highlightSlot);
  saveHeader(&header);

  // now save the data...
  int index = SAVE_DATA_POS + SAVE_SIZE*l_highlightSlot;
  // days...
  SaveByte(index++, g_saveArea.days&0xff);
  SaveByte(index++, (g_saveArea.days/256)&0xff);
  // who
  SaveByte(index++, g_saveArea.selectedChar);
  // "4" bytes, say

  // location_t g_objects
  for (i = 0; i < 128; i++) {
    SaveByte(index++, g_objects[i].x);
    SaveByte(index++, g_objects[i].y);
  } // 256 bytes

  // garrison_t g_garrisons
  for (i = 0; i < 128; i++) {
    SaveByte(index++, g_garrisons[i].size);
    SaveByte(index++, g_garrisons[i].owner);
  } // 256 bytes

  // chararmy_t g_charArmies
  for (i = 0; i < 128; i++) {
    SaveByte(index++, g_charArmies[i].size);
    SaveByte(index++, g_charArmies[i].lost);
    SaveByte(index++, g_charArmies[i].slew);
    SaveByte(index++, g_charArmies[i].race);
  } // 512 bytes

  // character_t g_characters (all values)
  for (i = 0; i < 128; i++) {
    SaveByte(index++, g_characters[i].location.x);
    SaveByte(index++, g_characters[i].location.y);
    SaveByte(index++, g_characters[i].time_looking);
    SaveByte(index++, g_characters[i].race_orders);
    SaveByte(index++, g_characters[i].flags);
    SaveByte(index++, g_characters[i].energy);
    SaveByte(index++, g_characters[i].fighting);
    SaveByte(index++, g_characters[i].foe);
    SaveByte(index++, g_characters[i].liege);
    SaveByte(index++, g_characters[i].reckless);
    SaveByte(index++, g_characters[i].object);
    SaveByte(index++, g_characters[i].attribute_good);
    SaveByte(index++, g_characters[i].attribute_bad);
    SaveByte(index++, g_characters[i].despondence);
  } // 14 * 128 = 1792 bytes

  // u8 g_map
  for (i = 0; i < 64*96; i++) {
    SaveByte(index++, g_map[i]);
  } // 6144 bytes
  
  // save the haveSeen map. Only 2 bits are used, so pack'em up
  i = 0;
  while (i < 64*96) {
    
    theByte = g_haveSeenMap[i++]&0x03;
    theByte |= (g_haveSeenMap[i++]<<2)&0x0C;
    theByte |= (g_haveSeenMap[i++]<<4)&0x30;
    theByte |= (g_haveSeenMap[i++]<<6)&0xC0;
    
    SaveByte(index++, theByte);
  } // 96*64 / 4 = 1536 bytes
  
  // save the guidance log:
  // assume 5 entries, 6 bytes per entry and 1 byte for size
  SaveByte(index++, g_historyDepth&0xff);
  for (i = 0; i < 5; i++) {
    SaveByte(index++, g_guidanceLog[i].whoOrWhat);
    SaveByte(index++, g_guidanceLog[i].receiver);
    SaveByte(index++, g_guidanceLog[i].location.x);
    SaveByte(index++, g_guidanceLog[i].location.y);
    SaveByte(index++, g_guidanceLog[i].ownerLocation.x);
    SaveByte(index++, g_guidanceLog[i].ownerLocation.y);
  } // another 32 bytes
  
  
  
  // total = 256+256+512+1792+6144 = 8960 
  // + 1536 + 32 = 10528
  
  addToTokenList(gameSaved);
  doSavedInfoScreen();

  return 0;
}

// load from slot l_highlightSlot
int loadData(void) {
  u8 theByte;
  int i;
  header_t header;
  loadHeader(&header);
  int bitmask = ((l_highlightSlot==0)?1:(1<<l_highlightSlot));
  if (!(header.isSaved & bitmask)) {
    // this slot not available
    return 0;
  }
  // now load the data...
  int index = SAVE_DATA_POS + SAVE_SIZE*l_highlightSlot;

  // days...
  g_saveArea.days = LoadByte(index++);
  g_saveArea.days |= 256 * LoadByte(index++);
  // who
  g_saveArea.selectedChar = LoadByte(index++);
  // "4" bytes
  
  // location_t g_objects
  for (i = 0; i < 128; i++) {
    g_objects[i].x = LoadByte(index++);
    g_objects[i].y = LoadByte(index++);
  } // 256 bytes

  // garrison_t g_garrisons
  for (i = 0; i < 128; i++) {
    g_garrisons[i].size = LoadByte(index++);
    g_garrisons[i].owner = LoadByte(index++);
  } // 256 bytes

  // chararmy_t g_charArmies
  for (i = 0; i < 128; i++) {
    g_charArmies[i].size = LoadByte(index++);
    g_charArmies[i].lost = LoadByte(index++);
    g_charArmies[i].slew = LoadByte(index++);
    g_charArmies[i].race = LoadByte(index++);
  } // 512 bytes

  // character_t g_characters (all values)
  for (i = 0; i < 128; i++) {
    g_characters[i].location.x = LoadByte(index++);
    g_characters[i].location.y = LoadByte(index++);
    g_characters[i].time_looking = LoadByte(index++);
    g_characters[i].race_orders = LoadByte(index++);
    g_characters[i].flags = LoadByte(index++);
    g_characters[i].energy = LoadByte(index++);
    g_characters[i].fighting = LoadByte(index++);
    g_characters[i].foe = LoadByte(index++);
    g_characters[i].liege = LoadByte(index++);
    g_characters[i].reckless = LoadByte(index++);
    g_characters[i].object = LoadByte(index++);
    g_characters[i].attribute_good = LoadByte(index++);
    g_characters[i].attribute_bad = LoadByte(index++);
    g_characters[i].despondence = LoadByte(index++);
  } // 13 * 128 = 1664 bytes

  // u8 g_map
  for (i = 0; i < 64*96; i++) {
    g_map[i] = LoadByte(index++);
  } // 6144 bytes
  
  // load the haveSeen map. Only 2 bits per entry, so unpack 'em 
  i = 0;
  while (i < 64*96) {
    theByte = LoadByte(index++);
    
    g_haveSeenMap[i++] = theByte&3;
    g_haveSeenMap[i++] = (theByte>>2)&3;
    g_haveSeenMap[i++] = (theByte>>4)&3;
    g_haveSeenMap[i++] = (theByte>>6)&3;
    
  } // 96*64 / 4 = 1536 bytes
  
  
  // load in the guidance log:
  // assume 5 entries, 6 bytes per entry and 1 byte for size
  g_historyDepth = LoadByte(index++);
  for (i = 0; i < 5; i++) {
    g_guidanceLog[i].whoOrWhat       = LoadByte(index++);
    g_guidanceLog[i].receiver        = LoadByte(index++);
    g_guidanceLog[i].location.x      = LoadByte(index++);
    g_guidanceLog[i].location.y      = LoadByte(index++);
    g_guidanceLog[i].ownerLocation.x = LoadByte(index++);
    g_guidanceLog[i].ownerLocation.y = LoadByte(index++);
  } // another 32 bytes
  
  
  return 1;
}

int deleteData(void) {
  
  header_t header;
  loadHeader(&header);
  int bitmask = ((l_highlightSlot==0)?1:(1<<l_highlightSlot));
  if (header.isSaved & bitmask) {
    header.isSaved &= ~bitmask;
    saveHeader(&header);
    addToTokenList(gameDeleted);
    doSavedInfoScreen();
  } 
  //else {
  //  // this slot not available
  //}
  return 0;
}

void waitKeys(void) {

  UpdateKeys();
  u16 delay = 10;
  while (1) {

    wait_vsync_int();
    UpdateKeys();

    if (KeyPressedNoBounce(KEY_UP,delay)) {
      if (l_highlightSlot) {
        l_highlightSlot--;
        selectOption(l_highlightSlot);
      }

    }

    if (KeyPressedNoBounce(KEY_DOWN,delay)) {
      if (l_highlightSlot < l_maxHighlight-1) {
        l_highlightSlot++;
        selectOption(l_highlightSlot);
      }
    }

    if (KeyPressedNoBounce(KEY_A,delay)) {
      if (actionPerformed()) {
        clearHighlighter();
        look();
        WaitKeyReleased(KEY_A);
        return;
      }
      WaitKeyReleased(KEY_A);
    }
    if (KeyPressedNoBounce(KEY_B,delay)) {
      // cancel..
      if (actionCancelled()) {
        look();
        WaitKeyReleased(KEY_B);
        // wait for let go?
        return;
      } // else carry on checking keys
      WaitKeyReleased(KEY_B);
    }
  }
}


int doMainSaveMenu(void);

void save(void) {
  addToTokenList(saveGame);
  doSavedInfoScreen();
  actionPerformed = saveData;
  actionCancelled = doMainSaveMenu;
  l_highlightSlot = 0;
  selectOption(l_highlightSlot);
}

void load(void) {
  addToTokenList(loadGame);
  doSavedInfoScreen();
  actionPerformed = loadData;
  actionCancelled = doMainSaveMenu;
  l_highlightSlot = 0;
  selectOption(l_highlightSlot);
}

void deleteSave(void) {
  addToTokenList(deleteGame);
  doSavedInfoScreen();
  actionPerformed = deleteData;
  actionCancelled = doMainSaveMenu;
  l_highlightSlot = 0;
  selectOption(l_highlightSlot);
}

int cancelMenu(void) {
  clearHighlighter();
  return 1;
}

void toggleSmoothLook(void) {
  g_panningOn = !g_panningOn;
  int tmpSel = l_highlightSlot;
  doMainSaveMenu();
  l_highlightSlot = tmpSel;
  selectOption(l_highlightSlot);
}


int makeSaveSelection(void) {

  switch (l_highlightSlot) {
    case 0: save(); break;
    case 1: load(); break;
    case 2: deleteSave(); break;
    case 3: toggleSmoothLook(); break;
    case 4: showMapScreen(); 
            doMainSaveMenu();
            break;
    case 5: showGuidanceLog(); 
            doMainSaveMenu();
            break;
    default : return 1;
  }
  return 0;
}

int doMainSaveMenu(void) {
  clearHighlighter();

  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('1');
  addToTokenList(saveGame);
  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('2');
  addToTokenList(loadGame);

  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('3');
  addToTokenList(deleteGame);

  addNewlineToken();
  if (g_panningOn) {
    addLiteralToken(ASCII_BOX_NIGHT);
  } else {
    addLiteralToken(ASCII_BOX_OK);
  }
  addLiteralToken('4');
  addToTokenList(smoothLooking);

  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('5');
  addToTokenList(theIcemark);
  
  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('6');
  addToTokenList(guidanceFound);
  
  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('7');
  addNameToken(0xd5); // return

  clear_text();

  print_tokens(2,2,0,0x1e,FONT_RED);
  // highlight one, then do the menu
  l_highlightSlot = 0;
  actionPerformed = makeSaveSelection;
  actionCancelled = cancelMenu;

  selectOption(l_highlightSlot);
  l_maxHighlight = SAVE_OPTIONS;
  return 0;
}



// call this from START or something
void showSaveMenu(void) {
  // clear + set palette
  setupInfoScreen();
  doMainSaveMenu();
  g_status.screen = MENU_SCREEN;

  // loop on keys - pressing A or B calls
  // actionPerformed (A) or actionCancelled (B)
  // if either returns true, then control returns here.
  // until then it keeps looping
  waitKeys();
}


